package llh.svs.core.services.schedule.support

import gaf3.core.exception.BusinessError
import llh.svs.core.services.schedule.entity.SvsScheduleTask
import org.slf4j.LoggerFactory
import org.springframework.beans.BeansException
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.context.ApplicationContext
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler
import org.springframework.scheduling.support.CronTrigger
import org.springframework.stereotype.Component
import org.springframework.util.Assert
import java.util.concurrent.ScheduledFuture

/**
 * 定时任务创建辅助类
 */
@Component
class ScheduleTaskWorker {

    @Autowired
    lateinit var ctx: ApplicationContext

    @Autowired
    lateinit var threadPoolTaskScheduler: ThreadPoolTaskScheduler

    /**
     * 创建新定时任务
     */
    fun startTask(entity: SvsScheduleTask) {
        log.debug("创建定时任务，className=${entity.taskClassName}，cron=${entity.cronExpression}")
        // 获取任务执行类
        val task = runCatching {
            ctx.getBean(Class.forName(entity.taskClassName))
        }.getOrElse {
            if (it is ClassNotFoundException) {
                log.error("${entity.taskClassName}类不存在")
            }
            if (it is BeansException) {
                log.error("${entity.taskClassName}未纳入bean管理")
            }
            log.error("配置${entity.taskClassName}任务异常,${it.localizedMessage}")
            throw BusinessError(BusinessError.ERR_SERVICE_FAULT, "任务启动失败")
        }
        Assert.isAssignable(ScheduledTask::class.java, task.javaClass, "定时任务类必须实现ScheduledOfTask接口")
        // 如果该任务正在执行，则取消
        scheduleMap[entity.taskClassName!!]?.cancel(true)
        // 添加定时任务
        val scheduleFuture =
            threadPoolTaskScheduler.schedule(task as Runnable, CronTrigger(entity.cronExpression!!))
        scheduleMap[entity.taskClassName!!] = scheduleFuture
    }

    /**
     * 停止定时任务
     */
    fun stopTask(entity: SvsScheduleTask) {
        log.debug("取消定时任务，className=${entity.taskClassName}")
        scheduleMap[entity.taskClassName!!]?.cancel(true)
    }

    /**
     * 手动执行一次任务
     */
    fun onceStart(entity: SvsScheduleTask) {
        log.debug("手动执行一次定时任务，className=${entity.taskClassName}")
        val task = runCatching {
            ctx.getBean(Class.forName(entity.taskClassName))
        }.getOrElse {
            if (it is ClassNotFoundException) {
                log.error("${entity.taskClassName}类不存在")
            }
            if (it is BeansException) {
                log.error("${entity.taskClassName}未纳入bean管理")
            }
            log.error("配置${entity.taskClassName}任务异常,${it.localizedMessage}")
            throw BusinessError(BusinessError.ERR_SERVICE_FAULT, "任务启动失败")
        }
        Assert.isAssignable(ScheduledTask::class.java, task.javaClass, "定时任务类必须实现ScheduledOfTask接口")
        val scheduleTask = task as ScheduledTask
        scheduleTask.execute()
    }

    companion object {
        private val log = LoggerFactory.getLogger(ScheduleTaskWorker::class.java)
        private val scheduleMap = mutableMapOf<String, ScheduledFuture<*>?>()
    }
}
