package llh.svs.core.services.sys.service

import gaf3.core.data.PagedData
import gaf3.core.exception.BusinessError
import llh.svs.core.services.sys.SystemConfiguration
import llh.svs.core.services.sys.domain.BackupFileResult
import llh.svs.core.services.task.entity.SvsTaskLog
import llh.svs.core.services.task.service.SvsTaskLogService
import llh.svs.core.services.task.support.TaskStatus
import llh.svs.core.services.task.support.TaskType
import llh.svs.core.util.LinuxCmdExecutor
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Value
import org.springframework.scheduling.annotation.Async
import org.springframework.stereotype.Service
import org.springframework.util.Assert
import java.io.File
import java.nio.file.Files
import java.nio.file.attribute.BasicFileAttributes
import java.time.LocalDateTime
import java.time.ZoneOffset
import java.time.format.DateTimeFormatter
import java.util.*

/**
 * 数据库备份回复相关操作
 */
@Service
class DBBackupService(val config: SystemConfiguration, val logService: SvsTaskLogService) {

    @Value("\${spring.datasource.url}")
    lateinit var url: String

    @Value("\${spring.datasource.username}")
    lateinit var username: String

    @Value("\${spring.datasource.password}")
    lateinit var password: String

    /**
     * 手动备份数据库
     */
    fun backupDatabase(file: String?) {
        if (config.dbBackup == null) throw BusinessError(BusinessError.ERR_BUSINESS, "数据库备份设置未配置")
        val filename = file ?: LocalDateTime.now().format(formatter)
        Assert.hasText(config.dbBackup!!.filepath, "数据库文件存放位置未配置")
        val database = url.substring(0, url.indexOf('?')).let {
            it.substring(it.lastIndexOf('/') + 1)
        }
        val dir = File(config.dbBackup!!.filepath!!)
        if (!dir.isDirectory) dir.mkdirs()
        val cmdString =
            "mysqldump -u$username -p$password --databases $database --lock-all-tables --result-file=${config.dbBackup!!.filepath}/${filename}.sql"
        runCatching {
            LinuxCmdExecutor.execute(cmdString)
        }.getOrElse {
            log.error("数据库备份出错")
            LinuxCmdExecutor.execute("rm -rf ${config.dbBackup!!.filepath}/${filename}.sql")
            throw BusinessError(BusinessError.ERR_BUSINESS, "数据库备份执行失败")
        }
    }

    /**
     * 数据库恢复，根据服务器的备份文件名
     */
    fun recovery(filename: String) {
        if (config.dbBackup == null) throw BusinessError(BusinessError.ERR_BUSINESS, "数据库备份设置未配置")
        Assert.hasText(config.dbBackup!!.filepath, "数据库文件存放位置未配置")
        val cmdString = "mysql -u$username -p$password < ${config.dbBackup!!.filepath}/${filename}"
        runCatching {
            LinuxCmdExecutor.execute(cmdString)
        }.getOrElse {
            log.error("数据库恢复出错")
            throw BusinessError(BusinessError.ERR_BUSINESS, "数据库恢复执行失败")
        }
    }

    /**
     * 恢复数据库并创建任务
     */
    fun createTask(file: File): SvsTaskLog {
        val acsTaskLog = SvsTaskLog().apply {
            this.taskType = TaskType.RECOVERYDB.type
            this.taskId = UUID.randomUUID().toString()
            this.status = TaskStatus.PENDING.status
            this.startTime = LocalDateTime.now()
            this.endTime = null
            this.progress = 0
            this.summary = null
        }
        //执行数据库恢复任务
        recovery(file, acsTaskLog)
        return acsTaskLog
    }

    /**
     * 根据用户上传的数据库脚本来进行数据库恢复
     */
    @Async
    fun recovery(file: File, acsTaskLog: SvsTaskLog) {
        if (!file.exists()) throw BusinessError(BusinessError.ERR_BUSINESS, "数据库备份临时文件不存在")
        val cmdString = "mysql -u$username -p$password < ${file.absoluteFile}"
        runCatching {
            LinuxCmdExecutor.execute(cmdString)
            logService.create(acsTaskLog)
        }.getOrElse {
            log.error("数据库恢复出错")
            logService.progress(
                taskId = acsTaskLog.taskId!!,
                status = TaskStatus.ERROR.status,
                summary = "数据库恢复失败：${it.message}"
            )
            throw BusinessError(BusinessError.ERR_BUSINESS, "数据库恢复执行失败")
        }
        //任务执行完成
        logService.progress(taskId = acsTaskLog.taskId!!, progress = 100, summary = "数据库恢复完成")
    }

    /**
     * 获取所有备份文件列表
     * @param skip 跳过记录数，从0开始
     * @param limit 每页条数
     */
    fun getAllBackupFile(skip: Int, limit: Int): PagedData<BackupFileResult> {
        if (config.dbBackup == null) throw BusinessError(BusinessError.ERR_BUSINESS, "数据库备份设置未配置")
        Assert.hasText(config.dbBackup!!.filepath, "备份文件存放路径未配置")
        val backupDir = File(config.dbBackup!!.filepath!!)
        if (!backupDir.isDirectory) backupDir.mkdirs()
        val list = backupDir.listFiles()?.filter { !it.name.contains(".cron") }?.map {
            BackupFileResult().apply {
                this.filename = it.name
                this.createtime = filenameFormatter.format(
                    LocalDateTime.ofInstant(
                        Files.readAttributes(
                            it.toPath(),
                            BasicFileAttributes::class.java
                        ).creationTime().toInstant(), ZoneOffset.of("+8")
                    )
                )
            }
        } ?: mutableListOf()
        val data = when {
            list.size < skip -> mutableListOf()
            list.size >= skip && list.size < skip + limit -> list.subList(skip, list.size)
            else -> list.subList(skip, skip + limit)
        }
        return PagedData(data = data, total = list.size)
    }

    /**
     * 根据文件名下载备份文件
     */
    fun downloadBackupFile(filename: String?): File {
        if (config.dbBackup == null) throw BusinessError(BusinessError.ERR_BUSINESS, "数据库备份设置未配置")
        Assert.hasText(config.dbBackup!!.filepath, "备份文件存放路径未配置")
        Assert.hasText(filename, "备份文件名不可为空")
        val file = File("${config.dbBackup!!.filepath!!}/${filename}")
        if (file.exists()) return file
        throw BusinessError(BusinessError.ERR_BUSINESS, "指定下载的备份文件不存在")
    }

    /**
     * 根据文件名删除备份文件
     */
    fun deleteBackupFile(filename: String?) {
        if (config.dbBackup == null) throw BusinessError(BusinessError.ERR_BUSINESS, "数据库备份设置未配置")
        Assert.hasText(config.dbBackup!!.filepath, "备份文件存放路径未配置")
        Assert.hasText(filename, "备份文件名不可为空")
        val file = File("${config.dbBackup!!.filepath!!}/${filename}")
        if (file.exists()) {
            file.delete()
        }
    }

    companion object {
        private val log = LoggerFactory.getLogger(DBBackupService::class.java)
        private val formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd_HH:mm:ss")
        private val filenameFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")
    }
}
