package llh.svs.core.services.task.service

import gaf3.core.data.PagedData
import gaf3.core.exception.BusinessError
import llh.svs.core.services.task.dao.SvsTaskLogDAO
import llh.svs.core.services.task.entity.SvsTaskLog
import llh.svs.core.services.task.support.TaskStatus
import llh.svs.core.services.task.support.TaskType
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.data.domain.Example
import org.springframework.data.domain.PageRequest
import org.springframework.data.domain.Sort
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.util.Assert
import java.time.LocalDateTime
import java.util.*
import javax.transaction.Transactional

@Service
class SvsTaskLogService {

    @Autowired
    lateinit var svsTaskLogDao: SvsTaskLogDAO

    /**
     * 根据id查找
     */
    fun findById(id: String): SvsTaskLog {
        return svsTaskLogDao.findByIdOrNull(id) ?: throw BusinessError(BusinessError.ERR_DATA_NOTEXIST, "任务不存在")
    }

    /**
     * 分页查询
     */
    fun find(filter: SvsTaskLog, skip: Int = 0, limit: Int = 20): PagedData<SvsTaskLog> {
        val page: Int = skip / limit
        val paged =
            svsTaskLogDao.findAll(Example.of(filter), PageRequest.of(page, limit, Sort.Direction.ASC, "startTime"))
        return PagedData(paged.content, paged.totalElements.toInt())
    }

    /**
     * 创建任务记录，表示任务已提交
     */
    fun create(data: SvsTaskLog): SvsTaskLog {
        Assert.hasText(data.taskType, "taskType不能为空")
        if (data.taskType.equals(TaskType.IMPORT.type, true) || data.taskType.equals(TaskType.EXPORT.type, true)) {
            Assert.hasText(data.dataType, "dataType不能为空")
            Assert.hasText(data.fileType, "fileType不能为空")
            Assert.hasText(data.filePath, "filePath不能为空")
        }
        data.taskId = data.taskId ?: UUID.randomUUID().toString()
        data.status = TaskStatus.PENDING.status
        data.startTime = LocalDateTime.now()
        data.endTime = null
        data.progress = 0
        data.summary = null
        return svsTaskLogDao.save(data)
    }

    /**
     * 修改任务进度
     */
    @Transactional(Transactional.TxType.REQUIRES_NEW)
    fun progress(taskId: String, progress: Int? = null, status: String? = null, summary: String? = null): SvsTaskLog {
        // 检查任务状态
        val task = svsTaskLogDao.findByIdOrNull(taskId) ?: throw BusinessError(BusinessError.ERR_DATA_NOTEXIST, "任务不存在")
        progress?.let {
            task.progress = progress
            task.status = if (progress == 100) TaskStatus.COMPLETED.status else TaskStatus.RUNNING.status
        }
        status?.let {
            task.status = status
        }
        summary?.let {
            task.summary = summary
        }

        if (task.status != TaskStatus.RUNNING.status) {
            task.endTime = LocalDateTime.now()
        }
        if (task.startTime == null) {
            task.startTime = LocalDateTime.now()
        }
        return svsTaskLogDao.save(task)
    }
}
